@@ -1,6 +1,6 @@
 /*
  * sadf: system activity data formatter
- * (C) 1999-2009 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2010 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -32,7 +32,8 @@
 #include "sa.h"
 #include "common.h"
 #include "ioconf.h"
-#include "prf_stats.h"
+#include "rndr_stats.h"
+#include "xml_stats.h"
 
 #ifdef USE_NLS
 # include <locale.h>
@@ -86,7 +87,7 @@ void usage(char *progname)
 		progname);
 
 	fprintf(stderr, _("Options are:\n"
-			  "[ -d | -D | -H | -p | -x ] [ -h ] [ -t ] [ -V ]\n"
+			  "[ -d | -D | -H | -p | -x ] [ -C ] [ -h ] [ -t ] [ -V ]\n"
 			  "[ -P { <cpu> [,...] | ALL } ] [ -s [ <hh:mm:ss> ] ] [ -e [ <hh:mm:ss> ] ]\n"
 			  "[ -- <sar_options> ]\n"));
 	exit(1);
@@ -193,10 +194,10 @@ void list_fields(unsigned int act_id)
 		if ((act_id != ALL_ACTIVITIES) && (act[i]->id != act_id))
 			continue;
 		
-		if (IS_SELECTED(act[i]->options) && (*act[i]->nr > 0)) {
+		if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0)) {
 			if (!HAS_MULTIPLE_OUTPUTS(act[i]->options)) {
 				printf(";%s", act[i]->hdr_line);
-				if ((*act[i]->nr > 1) && DISPLAY_HORIZONTALLY(flags)) {
+				if ((act[i]->nr > 1) && DISPLAY_HORIZONTALLY(flags)) {
 					printf("[...]");
 				}
 			}
@@ -206,7 +207,7 @@ void list_fields(unsigned int act_id)
 				for (hl = strtok(hline, "|"); hl; hl = strtok(NULL, "|"), msk <<= 1) {
 					if ((hl != NULL) && (act[i]->opt_flags & msk)) {
 						printf(";%s", hl);
-						if ((*act[i]->nr > 1) && DISPLAY_HORIZONTALLY(flags)) {
+						if ((act[i]->nr > 1) && DISPLAY_HORIZONTALLY(flags)) {
 							printf("[...]");
 						}
 					}
@@ -255,7 +256,7 @@ void write_mech_stats(int curr, unsigned long dt, unsigned long long itv,
 		if ((act_id != ALL_ACTIVITIES) && (act[i]->id != act_id))
 			continue;
 		
-		if (IS_SELECTED(act[i]->options) && (*act[i]->nr > 0)) {
+		if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0)) {
 			
 			if (NEEDS_GLOBAL_ITV(act[i]->options)) {
 				(*act[i]->f_render)(act[i], isdb, pre, curr, g_itv);
@@ -296,11 +297,11 @@ int write_parsable_stats(int curr, int reset, long *cnt, int use_tm_start,
 {
 	unsigned long long dt, itv, g_itv;
 	char cur_time[26];
-	static int cross_day = 0;
+	static int cross_day = FALSE;
 	static __nr_t cpu_nr = -1;
 	
 	if (cpu_nr < 0) {
-		cpu_nr = *act[get_activity_position(act, A_CPU)]->nr;
+		cpu_nr = act[get_activity_position(act, A_CPU)]->nr;
 	}
 
 	/*
@@ -322,7 +323,7 @@ int write_parsable_stats(int curr, int reset, long *cnt, int use_tm_start,
 	if (use_tm_start && record_hdr[!curr].ust_time &&
 	    (record_hdr[curr].ust_time > record_hdr[!curr].ust_time) &&
 	    (record_hdr[curr].hour < record_hdr[!curr].hour)) {
-		cross_day = 1;
+		cross_day = TRUE;
 	}
 
 	if (cross_day) {
@@ -366,24 +367,77 @@ int write_parsable_stats(int curr, int reset, long *cnt, int use_tm_start,
  * Display XML activity records
  *
  * IN:
- * @curr	Index in array for current sample statistics.
- * @tab		Number of tabulations to print.
- * @cpu_nr	Number of processors.
+ * @curr		Index in array for current sample statistics.
+ * @use_tm_start	Set to TRUE if option -s has been used.
+ * @use_tm_end		Set to TRUE if option -e has been used.
+ * @reset		Set to TRUE if last_uptime should be reinitialized
+ *			(used in next_slice() function).
+ * @tab			Number of tabulations to print.
+ * @cpu_nr		Number of processors.
+ *
+ * OUT:
+ * @cnt			Set to 0 to indicate that no other lines of stats
+ * 			should be displayed.
+ *
+ * RETURNS:
+ * 1 if stats have been successfully displayed.
  ***************************************************************************
  */
-void write_xml_stats(int curr, int tab, __nr_t cpu_nr)
+int write_xml_stats(int curr, int use_tm_start, int use_tm_end, int reset,
+		    long *cnt, int tab, __nr_t cpu_nr)
 {
 	int i;
 	unsigned long long dt, itv, g_itv;
 	char cur_time[XML_TIMESTAMP_LEN];
+	static int cross_day = FALSE;
 
 	/* Fill timestamp structure (rectime) for current record */
 	sadf_get_record_timestamp_struct(curr);
 
+	/*
+	 * Check time (1).
+	 * For this first check, we use the time interval entered on
+	 * the command line. This is equivalent to sar's option -i which
+	 * selects records at seconds as close as possible to the number
+	 * specified by the interval parameter.
+	 */
+	if (!next_slice(record_hdr[2].uptime0, record_hdr[curr].uptime0,
+			reset, interval))
+		/* Not close enough to desired interval */
+		return 0;
+
+	/* Check if we are beginning a new day */
+	if (use_tm_start && record_hdr[!curr].ust_time &&
+	    (record_hdr[curr].ust_time > record_hdr[!curr].ust_time) &&
+	    (record_hdr[curr].hour < record_hdr[!curr].hour)) {
+		cross_day = TRUE;
+	}
+
+	if (cross_day) {
+		/*
+		 * This is necessary if we want to properly handle something like:
+		 * sar -s time_start -e time_end with
+		 * time_start(day D) > time_end(day D+1)
+		 */
+		loctime.tm_hour += 24;
+	}
+
+	/* Check time (2) */
+	if (use_tm_start && (datecmp(&loctime, &tm_start) < 0))
+		/* it's too soon... */
+		return 0;
+
 	/* Get interval values */
 	get_itv_value(&record_hdr[curr], &record_hdr[!curr],
 		      cpu_nr, &itv, &g_itv);
 
+	/* Check time (3) */
+	if (use_tm_end && (datecmp(&loctime, &tm_end) > 0)) {
+		/* It's too late... */
+		*cnt = 0;
+		return 0;
+	}
+
 	dt = itv / HZ;
 	/* Correct rounding error for dt */
 	if ((itv % HZ) >= (HZ / 2)) {
@@ -396,12 +450,11 @@ void write_xml_stats(int curr, int tab, __nr_t cpu_nr)
         xprintf(tab, "<timestamp %s interval=\"%llu\">", cur_time, dt);
 	tab++;
 
-
 	/* Display XML statistics */
 	for (i = 0; i < NR_ACT; i++) {
 		
 		if (CLOSE_MARKUP(act[i]->options) ||
-		    (IS_SELECTED(act[i]->options) && (*act[i]->nr > 0))) {
+		    (IS_SELECTED(act[i]->options) && (act[i]->nr > 0))) {
 			
 			if (NEEDS_GLOBAL_ITV(act[i]->options)) {
 				(*act[i]->f_xml_print)(act[i], curr, tab, g_itv);
@@ -413,6 +466,8 @@ void write_xml_stats(int curr, int tab, __nr_t cpu_nr)
 	}
 
 	xprintf(--tab, "</timestamp>");
+
+	return 1;
 }
 
 /*
@@ -420,16 +475,23 @@ void write_xml_stats(int curr, int tab, __nr_t cpu_nr)
  * Display XML restart records
  *
  * IN:
- * @curr	Index in array for current sample statistics.
- * @tab		Number of tabulations to print.
+ * @curr		Index in array for current sample statistics.
+ * @use_tm_start	Set to TRUE if option -s has been used.
+ * @use_tm_end		Set to TRUE if option -e has been used.
+ * @tab			Number of tabulations to print.
  ***************************************************************************
  */
-void write_xml_restarts(int curr, int tab)
+void write_xml_restarts(int curr, int use_tm_start, int use_tm_end, int tab)
 {
 	char cur_time[64];
 
 	/* Fill timestamp structure for current record */
 	sadf_get_record_timestamp_struct(curr);
+	
+	/* The record must be in the interval specified by -s/-e options */
+	if ((use_tm_start && (datecmp(&loctime, &tm_start) < 0)) ||
+	    (use_tm_end && (datecmp(&loctime, &tm_end) > 0)))
+		return;
 
 	strftime(cur_time, 64, "date=\"%Y-%m-%d\" time=\"%H:%M:%S\"", &rectime);
 	xprintf(tab, "<boot %s/>", cur_time);
@@ -440,12 +502,14 @@ void write_xml_restarts(int curr, int tab)
  * Display XML COMMENT records
  *
  * IN:
- * @curr	Index in array for current sample statistics.
- * @tab		Number of tabulations to print.
- * @ifd		Input file descriptor.
+ * @curr		Index in array for current sample statistics.
+ * @use_tm_start	Set to TRUE if option -s has been used.
+ * @use_tm_end		Set to TRUE if option -e has been used.
+ * @tab			Number of tabulations to print.
+ * @ifd			Input file descriptor.
  ***************************************************************************
  */
-void write_xml_comments(int curr, int tab, int ifd)
+void write_xml_comments(int curr, int use_tm_start, int use_tm_end, int tab, int ifd)
 {
 	char cur_time[64];
 	char file_comment[MAX_COMMENT_LEN];
@@ -456,6 +520,11 @@ void write_xml_comments(int curr, int tab, int ifd)
 	/* Fill timestamp structure for current record */
 	sadf_get_record_timestamp_struct(curr);
 
+	/* The record must be in the interval specified by -s/-e options */
+	if ((use_tm_start && (datecmp(&loctime, &tm_start) < 0)) ||
+	    (use_tm_end && (datecmp(&loctime, &tm_end) > 0)))
+		return;
+
 	strftime(cur_time, 64, "date=\"%Y-%m-%d\" time=\"%H:%M:%S\"", &rectime);
 	xprintf(tab, "<comment %s com=\"%s\"/>", cur_time, file_comment);
 }
@@ -523,9 +592,9 @@ void sadf_print_special(int curr, int use_tm_start, int use_tm_end, int rtype, i
  * Display data file header.
  *
  * IN:
- * @dfile	Name of system activity data file
- * @file_magic	System activity file magic header
- * @file_hdr	System activity file standard header
+ * @dfile	Name of system activity data file.
+ * @file_magic	System activity file magic header.
+ * @file_hdr	System activity file standard header.
  ***************************************************************************
  */
 void display_file_header(char *dfile, struct file_magic *file_magic,
@@ -533,9 +602,9 @@ void display_file_header(char *dfile, struct file_magic *file_magic,
 {
 	int i, p;
 	static __nr_t cpu_nr = -1;
-	
+
 	if (cpu_nr < 0) {
-		cpu_nr = *act[get_activity_position(act, A_CPU)]->nr;
+		cpu_nr = act[get_activity_position(act, A_CPU)]->nr;
 	}
 	
 	printf(_("System activity data file: %s (%#x)\n"),
@@ -562,7 +631,14 @@ void display_file_header(char *dfile, struct file_magic *file_magic,
 		if ((p = get_activity_position(act, id_seq[i])) < 0) {
 			PANIC(id_seq[i]);
 		}
-		printf("%02d: %s\t(x%d)\n", act[p]->id,	act[p]->name, *act[p]->nr);
+		printf("%02d: %s\t(x%d)", act[p]->id, act[p]->name, act[p]->nr);
+		if (act[p]->f_count2 || (act[p]->nr2 > 1)) {
+			printf("\t(x%d)", act[p]->nr2);
+		}
+		if (act[p]->magic == ACTIVITY_MAGIC_UNKNOWN) {
+			printf(_("\t[Unknown activity format]"));
+		}
+		printf("\n");
 	}
 
 	exit(0);
@@ -681,7 +757,6 @@ void rw_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf,
 					(*cnt)--;
 				}
 			}
-
 			*reset = FALSE;
 		}
 	}
@@ -702,12 +777,13 @@ void rw_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf,
 void xml_display_loop(int ifd, struct file_activity *file_actlst)
 {
 	int curr, tab = 0, rtype;
-	int eosaf = TRUE;
+	int eosaf = TRUE, next, reset = FALSE;
+	long cnt = 1;
 	off_t fpos;
 	static __nr_t cpu_nr = -1;
 	
 	if (cpu_nr < 0) {
-		cpu_nr = *act[get_activity_position(act, A_CPU)]->nr;
+		cpu_nr = act[get_activity_position(act, A_CPU)]->nr;
 	}
 
 	/* Save current file position */
@@ -737,6 +813,7 @@ void xml_display_loop(int ifd, struct file_activity *file_actlst)
 				 */
 				read_file_stat_bunch(act, 0, ifd, file_hdr.sa_nr_act,
 						     file_actlst);
+				sadf_get_record_timestamp_struct(0);
 			}
 
 			if (!eosaf && (rtype == R_COMMENT)) {
@@ -750,9 +827,16 @@ void xml_display_loop(int ifd, struct file_activity *file_actlst)
 				}
 			}
 		}
-		while (!eosaf && ((rtype == R_RESTART) || (rtype == R_COMMENT)));
+		while (!eosaf && ((rtype == R_RESTART) || (rtype == R_COMMENT) ||
+			(tm_start.use && (datecmp(&loctime, &tm_start) < 0)) ||
+			(tm_end.use && (datecmp(&loctime, &tm_end) >=0))));
+
+		/* Save the first stats collected. Used for example in next_slice() function */
+		copy_structures(act, id_seq, record_hdr, 2, 0);
 
 		curr = 1;
+		cnt = count;
+		reset = TRUE;
 
 		if (!eosaf) {
 			do {
@@ -765,8 +849,17 @@ void xml_display_loop(int ifd, struct file_activity *file_actlst)
 					read_file_stat_bunch(act, curr, ifd, file_hdr.sa_nr_act,
 							     file_actlst);
 
-					write_xml_stats(curr, tab, cpu_nr);
-					curr ^= 1;
+					/* next is set to 1 when we were close enough to desired interval */
+					next = write_xml_stats(curr, tm_start.use, tm_end.use, reset,
+							&cnt, tab, cpu_nr);
+
+					if (next) {
+						curr ^= 1;
+						if (cnt > 0) {
+							cnt--;
+						}
+					}
+					reset = FALSE;
 				}
 
 				if (!eosaf && (rtype == R_COMMENT)) {
@@ -776,7 +869,28 @@ void xml_display_loop(int ifd, struct file_activity *file_actlst)
 					}
 				}
 			}
-			while (!eosaf && (rtype != R_RESTART));
+			while (cnt && !eosaf && (rtype != R_RESTART));
+
+			if (!cnt) {
+				/* Go to next Linux restart, if possible */
+				do {
+					eosaf = sa_fread(ifd, &record_hdr[curr], RECORD_HEADER_SIZE,
+							 SOFT_SIZE);
+					rtype = record_hdr[curr].record_type;
+					if (!eosaf && (rtype != R_RESTART) && (rtype != R_COMMENT)) {
+						read_file_stat_bunch(act, curr, ifd, file_hdr.sa_nr_act,
+								     file_actlst);
+					}
+					else if (!eosaf && (rtype == R_COMMENT)) {
+						/* Ignore COMMENT record */
+						if (lseek(ifd, MAX_COMMENT_LEN, SEEK_CUR) < MAX_COMMENT_LEN) {
+							perror("lseek");
+						}
+					}
+				}
+				while (!eosaf && (rtype != R_RESTART));
+			}
+			reset = TRUE;
 		}
 	}
 	while (!eosaf);
@@ -800,7 +914,7 @@ void xml_display_loop(int ifd, struct file_activity *file_actlst)
 						     file_actlst);
 			}
 			if (rtype == R_RESTART) {
-				write_xml_restarts(0, tab);
+				write_xml_restarts(0, tm_start.use, tm_end.use, tab);
 			}
 			else if (rtype == R_COMMENT) {
 				/* Ignore COMMENT record */
@@ -820,24 +934,27 @@ void xml_display_loop(int ifd, struct file_activity *file_actlst)
 	}
 
 	/* Last, process COMMENT entries to display comments */
-	xprintf(tab++, "<comments>");
-	do {
-		if ((eosaf = sa_fread(ifd, &record_hdr[0], RECORD_HEADER_SIZE,
-				      SOFT_SIZE)) == 0) {
+	if (DISPLAY_COMMENT(flags)) {
+		xprintf(tab++, "<comments>");
+		do {
+			if ((eosaf = sa_fread(ifd, &record_hdr[0], RECORD_HEADER_SIZE,
+				              SOFT_SIZE)) == 0) {
 
-			rtype = record_hdr[0].record_type;
-			if ((rtype != R_RESTART) && (rtype != R_COMMENT)) {
-				read_file_stat_bunch(act, 0, ifd, file_hdr.sa_nr_act,
-						     file_actlst);
-			}
-			if (rtype == R_COMMENT) {
-				write_xml_comments(0, tab, ifd);
+				rtype = record_hdr[0].record_type;
+				if ((rtype != R_RESTART) && (rtype != R_COMMENT)) {
+					read_file_stat_bunch(act, 0, ifd, file_hdr.sa_nr_act,
+							     file_actlst);
+				}
+				if (rtype == R_COMMENT) {
+					write_xml_comments(0, tm_start.use, tm_end.use,
+							   tab, ifd);
+				}
 			}
 		}
+		while (!eosaf);
+		xprintf(--tab, "</comments>");
 	}
-	while (!eosaf);
-	xprintf(--tab, "</comments>");
-
+	
 	xprintf(--tab, "</host>");
 	xprintf(--tab, "</sysstat>");
 }
@@ -891,7 +1008,8 @@ void main_display_loop(int ifd, struct file_activity *file_actlst)
 		/* Save the first stats collected. Will be used to compute the average */
 		copy_structures(act, id_seq, record_hdr, 2, 0);
 
-		reset = TRUE;	/* Set flag to reset last_uptime variable */
+		/* Set flag to reset last_uptime variable. Should be done after a LINUX RESTART record */
+		reset = TRUE;
 
 		/* Save current file position */
 		if ((fpos = lseek(ifd, 0, SEEK_CUR)) < 0) {
@@ -955,6 +1073,10 @@ void main_display_loop(int ifd, struct file_activity *file_actlst)
 					read_file_stat_bunch(act, curr, ifd, file_hdr.sa_nr_act,
 							     file_actlst);
 				}
+				else if (!eosaf && (rtype == R_COMMENT)) {
+					/* This was a COMMENT record: print it */
+					sadf_print_special(curr, tm_start.use, tm_end.use, R_COMMENT, ifd);
+				}
 			}
 			while (!eosaf && (rtype != R_RESTART));
 		}
@@ -1080,6 +1202,18 @@ int main(int argc, char **argv)
 			opt++;
 		}
 
+		else if (!strcmp(argv[opt], "-m")) {
+			if (argv[++opt] && sar_options) {
+				/* Parse sar's option -m */
+				if (parse_sar_m_opt(argv, &opt, act)) {
+					usage(argv[0]);
+				}
+			}
+			else {
+				usage(argv[0]);
+			}
+		}
+
 		else if (!strcmp(argv[opt], "-n")) {
 			if (argv[++opt] && sar_options) {
 				/* Parse sar's option -n */
@@ -1105,6 +1239,10 @@ int main(int argc, char **argv)
 
 					switch (*(argv[opt] + i)) {
 
+					case 'C':
+						flags |= S_F_COMMENT;
+						break;
+							
 					case 'd':
 						if (format && (format != S_O_DB_OPTION)) {
 							usage(argv[0]);
