@@ -102,7 +102,7 @@ void usage(char *progname)
 
 	print_usage_title(progname);
 	fprintf(stderr, _("Options are:\n"
-			  "[ -A ] [ -b ] [ -B ] [ -C ] [ -d ] [ -h ] [ -p ] [ -q ] [ -r ]\n"
+			  "[ -A ] [ -b ] [ -B ] [ -C ] [ -d ] [ -h ] [ -H ] [ -p ] [ -q ] [ -r ]\n"
 			  "[ -R ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -v ] [ -V ] [ -w ] [ -W ] [ -y ]\n"
 			  "[ -I { <int> [,...] | SUM | ALL | XALL } ] [ -P { <cpu> [,...] | ALL } ]\n"
 			  "[ -m { <keyword> [,...] | ALL } ] [ -n { <keyword> [,...] | ALL } ]\n"
@@ -127,13 +127,15 @@ void display_help(char *progname)
 	fprintf(stderr, _("\t-b\tI/O and transfer rate statistics\n"));
 	fprintf(stderr, _("\t-B\tPaging statistics\n"));
 	fprintf(stderr, _("\t-d\tBlock device statistics\n"));
+	fprintf(stderr, _("\t-H\tHugepages utilization statistics\n"));
 	fprintf(stderr, _("\t-I { <int> | SUM | ALL | XALL }\n"
 			  "\t\tInterrupts statistics\n"));
 	fprintf(stderr, _("\t-m { <keyword> [,...] | ALL }\n"
 			  "\t\tPower management statistics\n"
 			  "\t\tKeywords are:\n"
-			  "\t\tCPU\tCPU clock frequency\n"
+			  "\t\tCPU\tCPU instantaneous clock frequency\n"
 			  "\t\tFAN\tFans speed\n"
+			  "\t\tFREQ\tCPU average clock frequency\n"
 			  "\t\tIN\tVoltage inputs\n"
 			  "\t\tTEMP\tDevices temperature\n"));
 	fprintf(stderr, _("\t-n { <keyword> [,...] | ALL }\n"
@@ -217,7 +219,7 @@ void print_read_error(void)
  ***************************************************************************
  * Check that every selected activity actually belongs to the sequence list.
  * If not, then the activity should be unselected since it will not be sent
- * by sadc. An activity can be unsent if its number of items is null.
+ * by sadc. An activity can be not sent if its number of items is null.
  *
  * IN:
  * @act_nr	Size of sequence list.
@@ -508,8 +510,9 @@ void write_stats_startup(int curr)
 	record_hdr[!curr].ust_time    = record_hdr[curr].ust_time;
 
 	for (i = 0; i < NR_ACT; i++) {
-		if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0))
-			memset(act[i]->buf[!curr], 0, act[i]->msize * act[i]->nr);
+		if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0)) {
+			memset(act[i]->buf[!curr], 0, act[i]->msize * act[i]->nr * act[i]->nr2);
+		}
 	}
 	
 	flags |= S_F_SINCE_BOOT;
@@ -617,7 +620,7 @@ int sar_print_special(int curr, int use_tm_start, int use_tm_end, int rtype, int
 void read_sadc_stat_bunch(int curr)
 {
 	int i, p;
-	
+
 	/* Read record header (type is always R_STATS since it is read from sadc) */
 	if (sa_read(&record_hdr[curr], RECORD_HEADER_SIZE)) {
 		print_read_error();
@@ -630,8 +633,8 @@ void read_sadc_stat_bunch(int curr)
 		if ((p = get_activity_position(act, id_seq[i])) < 0) {
 			PANIC(1);
 		}
-		
-		if (sa_read(act[p]->buf[curr], act[p]->fsize * act[p]->nr)) {
+
+		if (sa_read(act[p]->buf[curr], act[p]->fsize * act[p]->nr * act[p]->nr2)) {
 			print_read_error();
 		}
 	}
@@ -800,13 +803,18 @@ void read_header_data(void)
 
 		p = get_activity_position(act, file_act.id);
 
-		if ((p < 0) || (act[p]->fsize != file_act.size) || !file_act.nr) {
+		if ((p < 0) || (act[p]->fsize != file_act.size)
+			    || !file_act.nr
+			    || !file_act.nr2
+			    || (act[p]->magic != file_act.magic)) {
+			/* Remember that we are reading data from sadc and not from a file... */
 			fprintf(stderr, _("Inconsistent input data\n"));
 			exit(3);
 		}
 
 		id_seq[i]   = file_act.id;	/* We necessarily have "i < NR_ACT" */
 		act[p]->nr  = file_act.nr;
+		act[p]->nr2 = file_act.nr2;
 	}
 
 	while (i < NR_ACT) {
@@ -889,7 +897,10 @@ void read_stats_from_file(char from_file[])
 			exit(2);
 		}
 
-		/* Read and write stats located between two possible Linux restarts */
+		/*
+		 * Read and write stats located between two possible Linux restarts.
+		 * Activities that should be displayed are saved in id_seq[] array.
+		 */
 		for (i = 0; i < NR_ACT; i++) {
 
 			if (!id_seq[i])
